@@ -139,6 +139,10 @@ module PgQuery
               @cte_names.concat(cte_names)
               statements.concat(cte_statements)
             end
+
+            if statement.select_stmt.into_clause
+              from_clause_items << { item: PgQuery::Node.new(range_var: statement.select_stmt.into_clause.rel), type: :ddl }
+            end
           # The following statements modify the contents of a table
           when :insert_stmt, :update_stmt, :delete_stmt
             value = statement.public_send(statement.node)
@@ -171,7 +175,12 @@ module PgQuery
             statements << statement.copy_stmt.query
           # The following statement types are DDL (changing table structure)
           when :alter_table_stmt
-            from_clause_items << { item: PgQuery::Node.new(range_var: statement.alter_table_stmt.relation), type: :ddl }
+            case statement.alter_table_stmt.objtype
+            when :OBJECT_INDEX # Index # rubocop:disable Lint/EmptyWhen
+              # ignore `ALTER INDEX index_name`
+            else
+              from_clause_items << { item: PgQuery::Node.new(range_var: statement.alter_table_stmt.relation), type: :ddl }
+            end
           when :create_stmt
             from_clause_items << { item: PgQuery::Node.new(range_var: statement.create_stmt.relation), type: :ddl }
           when :create_table_as_stmt
@@ -203,9 +212,9 @@ module PgQuery
             objects = statement.drop_stmt.objects.map do |obj|
               case obj.node
               when :list
-                obj.list.items.map { |obj2| obj2.string.str if obj2.node == :string }
+                obj.list.items.map { |obj2| obj2.string.sval if obj2.node == :string }
               when :string
-                obj.string.str
+                obj.string.sval
               end
             end
             case statement.drop_stmt.remove_type
@@ -216,7 +225,7 @@ module PgQuery
             when :OBJECT_FUNCTION
               # Only one function can be dropped in a statement
               obj = statement.drop_stmt.objects[0].object_with_args
-              @functions << { function: obj.objname.map { |f| f.string.str }.join('.'), type: :ddl }
+              @functions << { function: obj.objname.map { |f| f.string.sval }.join('.'), type: :ddl }
             end
           when :grant_stmt
             objects = statement.grant_stmt.objects
@@ -235,12 +244,12 @@ module PgQuery
             statements << statement.explain_stmt.query
           when :create_function_stmt
             @functions << {
-              function: statement.create_function_stmt.funcname.map { |f| f.string.str }.join('.'),
+              function: statement.create_function_stmt.funcname.map { |f| f.string.sval }.join('.'),
               type: :ddl
             }
           when :rename_stmt
             if statement.rename_stmt.rename_type == :OBJECT_FUNCTION
-              original_name = statement.rename_stmt.object.object_with_args.objname.map { |f| f.string.str }.join('.')
+              original_name = statement.rename_stmt.object.object_with_args.objname.map { |f| f.string.sval }.join('.')
               new_name = statement.rename_stmt.newname
               @functions += [
                 { function: original_name, type: :ddl },
@@ -280,7 +289,7 @@ module PgQuery
           when :func_call
             subselect_items.concat(next_item.func_call.args.to_ary)
             @functions << {
-              function: next_item.func_call.funcname.map { |f| f.string.str }.join('.'),
+              function: next_item.func_call.funcname.map { |f| f.string.sval }.join('.'),
               type: :call
             }
           when :case_expr
@@ -312,7 +321,8 @@ module PgQuery
               location: rangevar.location,
               schemaname: (rangevar.schemaname unless rangevar.schemaname.empty?),
               relname: rangevar.relname,
-              inh: rangevar.inh
+              inh: rangevar.inh,
+              relpersistence: rangevar.relpersistence
             }
             @aliases[rangevar.alias.aliasname] = table if rangevar.alias
           when :range_subselect
